@@ -38,7 +38,7 @@ from warnings import warn
 import numpy as np
 import pandas as pd
 import toml as toml
-from pandas.core.base import DataError
+from pandas.errors import DataError
 
 from qslib.plate_setup import PlateSetup
 from qslib.scpi_commands import AccessLevel, SCPICommand
@@ -50,12 +50,25 @@ from .data import FilterDataReading, FilterSet, df_from_readings
 from .machine import Machine
 from .processors import NormRaw, Processor
 from .protocol import Protocol, Stage, Step
+from .qs_is_protocol import QS_IOError
 from .rawquant_compat import _fdc_to_rawdata
 from .version import __version__
 
 if TYPE_CHECKING:  # pragma: no cover
     import matplotlib.pyplot as plt
 
+# Let's just assume all of these are problematic, for now.
+INVALID_NAME_RE = re.compile(r"[\[\]{}!/:;@&=+$,?#|\\]")
+
+
+def _safe_exp_name(name: str) -> str:
+    m = INVALID_NAME_RE.search(name)
+
+    if m:
+        raise ValueError(f"Invalid characters ({m[0]}) in run name: {name}.")
+
+    return name.replace(" ", "_")
+
 
 TEMPLATE_NAME = "ruo"
 
@@ -74,6 +87,20 @@ log = logging.getLogger(__name__)
 MachineReference = Union[str, Machine]
 
 
+@dataclass
+class AlreadyStartedError(ValueError):
+    """The experiment has already been started."""
+
+    name: str
+    state: str
+
+    def __str__(self) -> str:
+        return (
+            f"Experiment {self.name} is recorded as already having started."
+            f"Current state of the object is {self.state}."
+        )
+
+
 @dataclass
 class MachineError(Exception):
     """Base class for an error from a machine."""
@@ -112,13 +139,34 @@ class MachineBusyError(MachineError):
 
 @dataclass
 class AlreadyExistsError(MachineError):
+    "A run already exists with the same name."
+    name: str
+
+    def __str__(self) -> str:
+        return f"Run {self.name} exists."
+
+
+@dataclass
+class AlreadyExistsWorkingError(AlreadyExistsError):
     "A run already exists in uncollected (experiment:) with the same name."
     name: str
 
     def __str__(self) -> str:
         return (
             f"Run {self.name} exists in machine working directory. This may be caused by "
-            "a failed previous run.  TODO: write cleanup code."
+            'a failed previous run.  Start run with `overwrite="incomplete"` or `True` to overwrite.'
+        )
+
+
+@dataclass
+class AlreadyExistsCompleteError(AlreadyExistsError):
+    "A run already exists in uncollected (experiment:) with the same name."
+    name: str
+
+    def __str__(self) -> str:
+        return (
+            f"Run {self.name} is in completed run storage. This may be caused by "
+            "a failed previous run.  Start run with `overwrite=True` to overwrite."
         )
 
 
@@ -443,8 +491,8 @@ table, th, td {{
 
     @property
     def runtitle_safe(self) -> str:
-        """Run name with " " replaced by "_"."""
-        return self.name.replace(" ", "_")
+        """Run name with " " replaced by "_"; raises ValueError if name has other problematic characters."""
+        return _safe_exp_name(self.name)
 
     @property
     def rawdata(self) -> pd.DataFrame:
@@ -509,6 +557,7 @@ table, th, td {{
     def run(
         self,
         machine: MachineReference | None = None,
+        overwrite: Literal[True, False, "incomplete"] = False,
         require_exclusive: bool = False,
         require_drawer_check: bool = True,
     ) -> None:
@@ -519,13 +568,22 @@ table, th, td {{
         machine
             The machine to run on, by default None, in which case the machine
             associated with the run (if any) is used.
+        overwrite: bool or "incomplete", optional
+            Whether to overwrite files if a run with the same name already exists.
+            If "incomplete", only overwrite if the existing run is an incomplete
+            folder (in the experiments: working directory). By default False.
+        require_exclusive: bool, optional
+            Whether to require exclusive access to the machine, by default False.
+        require_drawer_check: bool, optional
+            Whether to ensure the drawer is closed before starting.  Note that
+            a close command will be sent regardless of this setting.  By default True.
 
         Raises
         ------
         MachineBusyError
             The machine isn't idle.
-        AlreadyExistsError
-            The machine already has a folder for this run in its working runs folder.
+        AlreadyExistsError (either AlreadyExistsWorkingError or AlreadyExistsCompleteError)
+            The machine already has a folder or eds file with this run name.
         """
         machine = self._ensure_machine(machine, needed_level=AccessLevel.Controller)
         log.info("Attempting to start %s on %s.", self.runtitle_safe, machine.host)
@@ -536,19 +594,37 @@ table, th, td {{
                 raise MachineBusyError(machine, x)
 
             if self.runstate != "INIT":
-                raise ValueError
+                raise AlreadyStartedError(self.runtitle_safe, self.runstate)
 
             with machine.at_access(AccessLevel.Controller, exclusive=require_exclusive):
+                # Check existence of previous run folder in experiments:
+                if self.runtitle_safe + "/" in machine.run_command("EXP:LIST?"):
+                    if not overwrite:
+                        raise AlreadyExistsWorkingError(machine, self.runtitle_safe)
+                    else:
+                        log.warning(
+                            f"Found incomplete run folder, deleting per overwrite={overwrite}."
+                        )
+                        machine.run_command(f"EXP:DELETE {self.runtitle_safe}")
+
+                # Check existence of previous completed run:
+                compl_runs = machine.list_files("public_run_complete:", verbose=False)
+                reds = f"public_run_complete:{self.runtitle_safe}.eds"
+                if reds in compl_runs:
+                    if (not overwrite) or (overwrite == "incomplete"):
+                        raise AlreadyExistsCompleteError(machine, self.runtitle_safe)
+                    else:
+                        log.warning(
+                            f"Found complete run, but overwriting per overwrite={overwrite}."
+                        )
+                        machine.run_command(f"FILE:REMOVE {reds}")
+
                 log.debug("Powering on machine and ensuring drawer/cover is closed.")
                 # Ensure machine state and power.
                 machine.power = True
 
                 machine.drawer_close(lower_cover=True, check=require_drawer_check)
 
-                # Check existence of previous run folder
-                if self.runtitle_safe + "/" in machine.run_command("EXP:LIST?"):
-                    raise AlreadyExistsError(machine, self.runtitle_safe)
-
                 log.debug("Creating experiment folder %s.", self.runtitle_safe)
                 machine.run_command(f"EXP:NEW {self.runtitle_safe} {TEMPLATE_NAME}")
 
@@ -725,7 +801,6 @@ table, th, td {{
                         b.write(machine.read_file(f["path"]))
                 elif log_method == "eval":
                     with open(sdspath, "ab+") as b:
-
                         # Seeking to the end of the file gives us the size
                         curpos = b.seek(0, 1)
 
@@ -875,6 +950,13 @@ table, th, td {{
                 open(self._sdspath("qsl-tcprotocol.xml"), "rb").read(),
             )
 
+    #     def change_plate_setup(self, new_plate_setup: Optional[PlateSetup], machine: MachineReference | None = None,
+    # ):
+    #         """
+    #         Change the plate setup for the experiment, including on the machine where it is running.
+
+    #         """
+
     def save_file(
         self,
         path_or_stream: str | os.PathLike[str] | io.BytesIO | IO[bytes] = ".",
@@ -976,6 +1058,10 @@ table, th, td {{
         else:
             self.name = _nowuuid()
 
+        # Ensure that name is safe for use as a filename:
+        # this will raise a ValueError for us if it isn't.
+        self.runtitle_safe
+
         if protocol is None:
             self.protocol = Protocol([Stage([Step(60, 25)])])
         else:
@@ -1180,15 +1266,18 @@ table, th, td {{
         machine = exp._ensure_machine(machine)
 
         with machine.ensured_connection():
-            crt = name
-
             if move:
                 raise NotImplementedError
 
-            if not crt:
-                raise ValueError("Nothing is currently running.")
-
-            z = machine.read_dir_as_zip(crt, leaf="EXP")
+            try:
+                z = machine.read_dir_as_zip(_safe_exp_name(name), leaf="EXP")
+            except QS_IOError:
+                try:
+                    z = machine.read_dir_as_zip(name, leaf="EXP")
+                except QS_IOError:
+                    raise ValueError(
+                        f"Could not find experiment {name} in uncollect runs on {machine}."
+                    )
 
             z.extractall(exp._dir_base)
 
@@ -1198,12 +1287,15 @@ table, th, td {{
 
     @classmethod
     def from_machine_storage(cls, machine: MachineReference, name: str) -> Experiment:
-        """Create an experiment from the one currently running on a machine.
+        """Create an experiment from the machine's storage.
 
         Parameters
         ----------
         machine : Machine
-            the machine to connect to
+            the machine to connect to.
+
+        name: str
+            the name of the run to collect.
 
         Returns
         -------
@@ -1215,10 +1307,20 @@ table, th, td {{
         machine = exp._ensure_machine(machine)
 
         with machine.ensured_connection():
-            try:
-                o = machine.read_file(name + ".eds", context="public_run_complete")
-            except FileNotFoundError:
-                o = machine.read_file(name, context="public_run_complete")
+            o = None
+            for possible_name in [
+                _safe_exp_name(name) + ".eds",
+                _safe_exp_name(name),
+                name + ".eds",
+                name,
+            ]:
+                try:
+                    o = machine.read_file(possible_name, context="public_run_complete")
+                    break
+                except QS_IOError:
+                    continue
+            if o is None:
+                raise FileNotFoundError(f"Could not find {name} on {machine.host}.")
 
             z = zipfile.ZipFile(io.BytesIO(o))
 
@@ -1250,13 +1352,22 @@ table, th, td {{
         if isinstance(machine, str):
             machine = Machine(machine)
 
+        safename = _safe_exp_name(name)
+
         with machine.ensured_connection():
-            if name == machine.current_run_name:
+            if machine.current_run_name in [safename, name]:
                 exp = cls.from_running(machine)
-            elif name in machine.list_runs_in_storage():
+                return exp
+
+            storage_runs = machine.list_runs_in_storage()
+            if (name in storage_runs) or (safename in storage_runs):
                 exp = cls.from_machine_storage(machine, name)
-            elif name + "/" in machine.list_files("", verbose=False, leaf="EXP"):
+                return exp
+
+            exp_runs = machine.list_files("", verbose=False, leaf="EXP")
+            if ((name + "/") in exp_runs) or ((safename + "/") in exp_runs):
                 exp = cls.from_uncollected(machine, name)
+
             else:
                 raise FileNotFoundError(f"Could not find run {name} on {machine.host}.")
         return exp
@@ -1491,10 +1602,10 @@ table, th, td {{
         if self.activestarttime:
             self.stages["start_seconds"] = (
                 self.stages["start_time"] - self.activestarttime
-            ).astype("timedelta64[s]")
+            ).astype("timedelta64[ms]").astype("int64") / 1000
             self.stages["end_seconds"] = (
                 self.stages["end_time"] - self.activestarttime
-            ).astype("timedelta64[s]")
+            ).astype("timedelta64[ms]").astype("int64") / 1000
 
         tt = []
 
@@ -2254,7 +2365,9 @@ table, th, td {{
             else:
                 annotate_frac = 0.05
 
-            for _, s in self.stages.iloc[1:-1].iterrows():
+            for _, s in self.stages.iterrows():
+                if s.stage == "PRERUN" or s.stage == "POSTRUN":
+                    continue
                 xtrans = ax.get_xaxis_transform()
                 ax.axvline(
                     s.start_seconds / 3600.0,
