@@ -9,7 +9,7 @@ import pytest
 
 import qslib.protocol as tc
 from qslib import Experiment
-from qslib.protocol import UR, Exposure, FilterSet, Protocol, Stage, Step
+from qslib.protocol import Q_, UR, Exposure, FilterSet, Protocol, Stage, Step
 from qslib.scpi_commands import SCPICommand  # noqa
 
 PROTSTRING = """PROTOCOL -volume=30 -runmode=standard testproto <multiline.protocol>
@@ -242,3 +242,55 @@ def test_exposure_roundtrip():
     e = Exposure([(FilterSet(1, 4), [500, 2000])])
     s = e.to_scpicommand().to_string()
     assert SCPICommand.from_string(s).specialize() == e
+
+
+def test_stepped_ramp_multi_same_increment():
+    p1 = Protocol(
+        [
+            Stage.stepped_ramp(
+                [50, 50, 50, 40, 40, 40], [40, 40, 40, 30, 30, 30], "11 min"
+            ),
+            Stage.stepped_ramp(
+                [40, 40, 40, 30, 30, 30],
+                [30, 30, 30, 20, 20, 20.0],
+                "10 min",
+                start_increment=True,
+            ),
+            # Stage.stepped_ramp(None, [20,20,20,10,10,10], "9 min", start_increment=True), # FIXME: implement this
+        ]
+    )
+
+    s = str(p1)
+
+    p1.to_scpicommand()
+
+    assert (
+        (p1.dataframe.loc[:, "temperature_1":"temperature_6"].diff()[1:] == -1)
+        .all()
+        .all()
+    )
+    assert (p1.dataframe.start_time.diff()[1:] == 60.625).all()
+    assert (p1.dataframe.end_time.diff()[1:] == 60.625).all()
+
+
+def test_delta_unit_conversion():
+    assert (
+        Stage.stepped_ramp(
+            "50 °C", "20 °C", total_time="30 min", temperature_step="2 °C"
+        )
+        == Stage.stepped_ramp(
+            "50 °C", "20 °C", total_time="30 min", temperature_step="2 delta_degC"
+        )
+        == Stage.stepped_ramp(
+            "50 °C", "20 °C", total_time="30 min", temperature_step="3.6 degF"
+        )
+    )
+
+
+def test_hold():
+    h = Stage.hold_at("60 °C", "1 hour", "10 minutes")
+    assert h == Stage(Step("10 min", "60 °C"), 6)
+
+    assert Stage.hold_at("50 °C", total_time="1 hour").steps[0].duration_at_cycle(
+        0
+    ) == Q_("1 hour")
